// Fig 18.6: FileSearchForm.cs
// Using regular expressions to determine file types.
using System;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;
using System.Collections.Specialized;

// uses regular expressions to determine file types
public partial class FileSearchForm : Form
{
   string currentDirectory = Directory.GetCurrentDirectory();
   string[] directoryList; // subdirectories
   string[] fileArray;

   // store extensions found and number found
   NameValueCollection found = new NameValueCollection();

   // parameterless constructor
   public FileSearchForm()
   {
      InitializeComponent();
   } // end constructor

   // invoked when user types in text box
   private void inputTextBox_KeyDown( object sender, KeyEventArgs e )
   {
      // determine whether user pressed Enter
      if ( e.KeyCode == Keys.Enter )
         searchButton_Click( sender, e );
   } // end method inputTextBox_KeyDown

   // invoked when user clicks "Search Directory" button
   private void searchButton_Click( object sender, EventArgs e )
   {
      // check for user input; default is current directory
      if ( inputTextBox.Text != "" )
      {
         // verify that user input is valid directory name
         if ( Directory.Exists( inputTextBox.Text ) )
         {
            currentDirectory = inputTextBox.Text;

            // reset input text box and update display
            directoryLabel.Text = "Current Directory:" +
               "\r\n" + currentDirectory;
         } // end if
         else
         {
            // show error if user does not specify valid directory
            MessageBox.Show( "Invalid Directory", "Error",
               MessageBoxButtons.OK, MessageBoxIcon.Error );
         } // end else
      } // end if

      // clear text boxes
      inputTextBox.Text = "";
      outputTextBox.Text = "";

      SearchDirectory( currentDirectory );   // search directory

      // summarize and print results
      foreach ( string current in found )
      {
         outputTextBox.Text += "* Found " +
            found[ current ] + " " + current + " files.\r\n";
      } // end foreach

      // clear output for new search
      found.Clear();
   } // end method searchButton_Click

   // search directory using regular expression
   private void SearchDirectory( string currentDirectory )
   {
      // for file name without directory path
      try
      {
         string fileName = "";

         // regular expression for extensions matching pattern
         Regex regularExpression = new Regex(
            @"[a-zA-Z0-9]+\.(?<extension>\w+)" );

         // stores regular-expression match result
         Match matchResult;

         string fileExtension; // holds file extensions

         // number of files with given extension in directory
         int extensionCount;

         // get directories
         directoryList = Directory.GetDirectories( currentDirectory );

         // get list of files in current directory
         fileArray = Directory.GetFiles( currentDirectory );

         // iterate through list of files
         foreach ( string myFile in fileArray )
         {
            // remove directory path from file name
            fileName = myFile.Substring( myFile.LastIndexOf( @"\" ) + 1 );

            // obtain result for regular-expression search
            matchResult = regularExpression.Match( fileName );

            // check for match
            if ( matchResult.Success )
               fileExtension = matchResult.Result( "${extension}" );
            else
               fileExtension = "[no extension]";

            // store value from container
            if ( found[ fileExtension ] == null )
               found.Add( fileExtension, "1" );
            else
            {
               extensionCount = Int32.Parse( found[ fileExtension ] ) + 1;
               found[ fileExtension ] = extensionCount.ToString();
            } // end else

            // search for backup( .bak ) files
            if ( fileExtension == "bak" )
            {
               // prompt user to delete ( .bak ) file
               DialogResult result =
                  MessageBox.Show( "Found backup file " +
                  fileName + ". Delete?", "Delete Backup",
                  MessageBoxButtons.YesNo, MessageBoxIcon.Question );

               // delete file if user clicked 'yes'
               if ( result == DialogResult.Yes )
               {
                  File.Delete( myFile );
                  extensionCount = Int32.Parse( found[ "bak" ] ) - 1;
                  found[ "bak" ] = extensionCount.ToString();
               } // end if
            } // end if
         } // end foreach

         // recursive call to search files in subdirectory
         foreach ( string myDirectory in directoryList )
            SearchDirectory( myDirectory );
      } // end try
      // handle exception if files have unauthorized access
      catch ( UnauthorizedAccessException )
      {
         MessageBox.Show( "Some files may not be visible" +
            " due to permission settings", "Warning",
            MessageBoxButtons.OK, MessageBoxIcon.Information );
      } // end catch
   } // end method SearchDirectory
} // end class FileSearchForm



/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/