// Fig. 19.22: XmlReaderTest.cs
// Reading an XML document.
using System;
using System.Windows.Forms;
using System.Xml;

namespace XmlReaderTest
{
   public partial class XmlReaderTestForm : Form
   {
      public XmlReaderTestForm()
      {
         InitializeComponent();
      } // end constructor

      // read XML document and display its content
      private void XmlReaderTestForm_Load( object sender, EventArgs e )
      {
         // create the XmlReader object
         XmlReaderSettings settings = new XmlReaderSettings();
         XmlReader reader = XmlReader.Create( "article.xml", settings );

         int depth = -1; // tree depth is -1, no indentation

         while ( reader.Read() ) // display each node's content
         {
            switch ( reader.NodeType )
            {
               case XmlNodeType.Element: // XML Element, display its name
                  depth++; // increase tab depth
                  TabOutput( depth ); // insert tabs
                  txtOutput.Text += "<" + reader.Name + ">\r\n";

                  // if empty element, decrease depth
                  if ( reader.IsEmptyElement )
                     depth--;
                  break;
               case XmlNodeType.Comment: // XML Comment, display it
                  TabOutput( depth ); // insert tabs
                  txtOutput.Text += "<!--" + reader.Value + "-->\r\n";
                  break;
               case XmlNodeType.Text: // XML Text, display it
                  TabOutput( depth ); // insert tabs
                  txtOutput.Text += "\t" + reader.Value + "\r\n";
                  break;

               // XML XMLDeclaration, display it
               case XmlNodeType.XmlDeclaration: 
                  TabOutput( depth ); // insert tabs
                  txtOutput.Text += "<?" + reader.Name + " " +
                     reader.Value + "?>\r\n";
                  break;
               case XmlNodeType.EndElement: // XML EndElement, display it
                  TabOutput( depth ); // insert tabs
                  txtOutput.Text += "</" + reader.Name + ">\r\n";
                  depth--; // decrement depth
                  break;
            } // end switch
         } // end while
      } // end method XmlReaderTextForm_Load

      // insert tabs
      private void TabOutput( int number )
      {
         for ( int i = 0; i < number; i++ )
            txtOutput.Text += "\t";
      } // end method TabOutput
   } // end class XmlReaderTestForm
} // end namespace XmlReaderTest

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/