// Fig. 19.23: XmlDom.cs
// Demonstrates DOM tree manipulation.
using System;
using System.Windows.Forms;
using System.Xml;

namespace XmlDom
{
   public partial class XmlDomForm : Form
   {
      public XmlDomForm()
      {
         InitializeComponent();
      } // end constructor

      private TreeNode tree; // TreeNode reference

      // initialize instance variables
      private void XmlDomForm_Load( object sender, EventArgs e )
      {
         // create Xml ReaderSettings and
         // set the IgnoreWhitespace property
         XmlReaderSettings settings = new XmlReaderSettings();
         settings.IgnoreWhitespace = true;

         // create XmlReader object
         XmlReader reader = XmlReader.Create( "letter.xml", settings );

         tree = new TreeNode(); // instantiate TreeNode

         tree.Text = "letter.xml"; // assign name to TreeNode
         treeXml.Nodes.Add( tree ); // add TreeNode to TreeView control
         BuildTree( reader, tree ); // build node and tree hierarchy
      } // end method XmlDomForm_Load

      // construct TreeView based on DOM tree
      private void BuildTree( XmlReader reader, TreeNode treeNode )
      {
         // treeNode to add to existing tree
         TreeNode newNode = new TreeNode();

         while ( reader.Read() )
         {
            // build tree based on node type
            switch ( reader.NodeType )
            {
               // if Text node, add its value to tree
               case XmlNodeType.Text:
                  newNode.Text = reader.Value;
                  treeNode.Nodes.Add( newNode );
                  break;
               case XmlNodeType.EndElement: // if EndElement, move up tree
                  treeNode = treeNode.Parent;
                  break;

               // if new element, add name and traverse tree
               case XmlNodeType.Element:

                  // determine if element contains content
                  if ( !reader.IsEmptyElement )
                  {
                     // assign node text, add newNode as child
                     newNode.Text = reader.Name;
                     treeNode.Nodes.Add( newNode );

                     // set treeNode to last child
                     treeNode = newNode;
                  } // end if
                  else // do not traverse empty elements
                  {
                     // assign NodeType string to newNode
                     // and add it to tree
                     newNode.Text = reader.NodeType.ToString();
                     treeNode.Nodes.Add( newNode );
                  } // end else
                  break;
               default: // all other types, display node type
                  newNode.Text = reader.NodeType.ToString();
                  treeNode.Nodes.Add( newNode );
                  break;
            } // end switch

            newNode = new TreeNode();
         } // end while

         // update TreeView control
         treeXml.ExpandAll(); // expand tree nodes in TreeView
         treeXml.Refresh(); // force TreeView to update
      } // end method BuildTree
   } // end class XmlDomForm
} // end namespace XmlDom

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/