// Fig. 19.28: ValidationTest.cs
// Validating XML documents against schemas.
using System;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Schema; // contains XmlSchemaSet class

namespace ValidationTest
{
   public partial class ValidationTestForm : Form
   {
      public ValidationTestForm()
      {
         InitializeComponent();
      } // end constructor

      private XmlSchemaSet schemas; // schemas to validate against
      private bool valid = true; // validation result

      // handle Validate Button Click event
      private void btnValidate_Click( object sender, EventArgs e )
      {
         schemas = new XmlSchemaSet(); // create the XmlSchemaSet class

         // add the schema to the collection
         schemas.Add( "http://www.deitel.com/booklist", "book.xsd" );

         // set the validation settings
         XmlReaderSettings settings = new XmlReaderSettings();
         settings.ValidationType = ValidationType.Schema;
         settings.Schemas = schemas;
         settings.ValidationEventHandler += ValidationError;

         // create the XmlReader object
         XmlReader reader = 
            XmlReader.Create( cboFiles.Text, settings );

         // parse the file
         while ( reader.Read() ); // empty body
         
         if ( valid ) // check validation result
         {
            lblConsole.Text = "Document is valid";
         } // end if

         valid = true; // reset variable
         reader.Close(); // close reader stream 
      } // end method btnValidate_Click

      // event handler for validation error
      private void ValidationError( object sender ,
         ValidationEventArgs arguments )
      {
         lblConsole.Text = arguments.Message;
         valid = false; // validation failed
      } // end method ValidationError
   } // end class ValidatorTestForm
} // end namespace ValidatorTest

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/