// Fig. 23.4: PacketClient.cs
// Set up a client that sends packets to a server and receives 
// packets from a server.
using System;
using System.Windows.Forms;
using System.Net;        
using System.Net.Sockets;
using System.Threading;  

public partial class PacketClientForm : Form
{
   public PacketClientForm()
   {
      InitializeComponent();
   } // end constructor

   private UdpClient client;       
   private IPEndPoint receivePoint;

   // initialize variables and thread for receiving packets
   private void PacketClientForm_Load( object sender, EventArgs e )
   {
      receivePoint = new IPEndPoint( new IPAddress( 0 ), 0 );
      client = new UdpClient( 50001 );                        
      Thread thread =                                        
         new Thread( new ThreadStart( WaitForPackets ) );    
      thread.Start();                                        
   } // end method PacketClientForm_Load

   // shut down the client
   private void PacketClientForm_FormClosing( object sender, 
      FormClosingEventArgs e )
   {
      System.Environment.Exit( System.Environment.ExitCode );
   } // end method PacketClientForm_FormClosing

   // delegate that allows method DisplayMessage to be called
   // in the thread that creates and maintains the GUI       
   private delegate void DisplayDelegate( string message );  

   // method DisplayMessage sets displayTextBox's Text property
   // in a thread-safe manner
   private void DisplayMessage( string message )
   {
      // if modifying displayTextBox is not thread safe
      if ( displayTextBox.InvokeRequired )
      {
         // use inherited method Invoke to execute DisplayMessage
         // via a delegate                                       
         Invoke( new DisplayDelegate( DisplayMessage ),          
            new object[] { message } );                          
      } // end if
      else // OK to modify displayTextBox in current thread
         displayTextBox.Text += message;
   } // end method DisplayMessage

   // send a packet
   private void inputTextBox_KeyDown( object sender, KeyEventArgs e )
   {
      if ( e.KeyCode == Keys.Enter )
      {
         // create packet (datagram) as string
         string packet = inputTextBox.Text;   
         displayTextBox.Text +=
            "\r\nSending packet containing: " + packet;

         // convert packet to byte array                             
         byte[] data = System.Text.Encoding.ASCII.GetBytes( packet );

         // send packet to server on port 50000               
         client.Send( data, data.Length, "127.0.0.1", 50000 );
         displayTextBox.Text += "\r\nPacket sent\r\n";
         inputTextBox.Clear();
      } // end if
   } // end method inputTextBox_KeyDown

   // wait for packets to arrive
   public void WaitForPackets()
   {
      while ( true )
      {
         // receive byte array from server                
         byte[] data = client.Receive( ref receivePoint );

         // output packet data to TextBox
         DisplayMessage( "\r\nPacket received:" +
            "\r\nLength: " + data.Length + "\r\nContaining: " + 
            System.Text.Encoding.ASCII.GetString( data ) + "\r\n" );
      } // end while
   } // end method WaitForPackets
} // end class PacketClientForm

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/