// Fig. 23.13: WeatherClient.cs
// Client that uses .NET remoting to retrieve a weather report.
using System;
using System.Collections;                   
using System.Drawing;                       
using System.Windows.Forms;                 
using System.Runtime.Remoting;              
using System.Runtime.Remoting.Channels;     
using System.Runtime.Remoting.Channels.Http;
using Weather;                              

public partial class WeatherClientForm : Form
{
   public WeatherClientForm()
   {
      InitializeComponent();
   } // end constructor

   // retrieve weather data
   private void WeatherClientForm_Load( object sender, EventArgs e )
   {
      // setup HTTP channel, does not need to provide a port number
      HttpChannel channel = new HttpChannel();                     
      ChannelServices.RegisterChannel( channel, false );                  

      // obtain a proxy for an object that implements interface Report
      Report info = ( Report ) RemotingServices.Connect(              
         typeof( Report ), "http://localhost:50000/Report" );         

      // retrieve an ArrayList of CityWeather objects
      ArrayList cities = info.Reports;               

      // create array and populate it with every Label
      Label[] cityLabels = new Label[ 43 ];
      int labelCounter = 0; 

      foreach ( Control control in displayPanel.Controls )
      {
         if ( control is Label )
         {
            cityLabels[ labelCounter ] = ( Label ) control;
            ++labelCounter; // increment Label counter
         } // end if
      } // end foreach

      // create Hashtable and populate with all weather conditions
      Hashtable weather = new Hashtable();                        
      weather.Add( "SUNNY", "sunny" );
      weather.Add( "PTCLDY", "pcloudy" );
      weather.Add( "CLOUDY", "mcloudy" );
      weather.Add( "MOCLDY", "mcloudy" );
      weather.Add( "TSTRMS", "rain" );
      weather.Add( "RAIN", "rain" );
      weather.Add( "SNOW", "snow" );
      weather.Add( "VRYHOT", "vryhot" );
      weather.Add( "FAIR", "fair" );
      weather.Add( "RNSNOW", "rnsnow" );
      weather.Add( "SHWRS", "showers" );
      weather.Add( "WINDY", "windy" );
      weather.Add( "NOINFO", "noinfo" );
      weather.Add( "MISG", "noinfo" );
      weather.Add( "DRZL", "rain" );
      weather.Add( "HAZE", "noinfo" );
      weather.Add( "SMOKE", "mcloudy" );
      weather.Add( "SNOWSHWRS", "snow" );
      weather.Add( "FLRRYS", "snow" );
      weather.Add( "FOG", "noinfo" );

      // create the font for the text output                   
      Font font = new Font( "Courier New", 8, FontStyle.Bold );

      // for every city
      for ( int i = 0; i < cities.Count; i++ )
      {
         // use array cityLabels to find the next Label
         Label currentCity = cityLabels[ i ];          

         // use ArrayList cities to find the next CityWeather object
         CityWeather city = ( CityWeather ) cities[ i ];            

         // set current Label's image to image                          
         // corresponding to the city's weather condition -             
         // find correct image name in Hashtable weather                
         currentCity.Image = new Bitmap( @"images\" +                   
            weather[ city.Description.Trim() ] + ".png" );       
         currentCity.Font = font; // set font of Label                  
         currentCity.ForeColor = Color.White; // set text color of Label

         // set Label's text to city name and temperature              
         currentCity.Text = "\r\n " + city.CityName + city.Temperature;
      } // end for
   } // end method WeatherClientForm_Load
} // end class WeatherClientForm

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/