// Fig. 23.11: ReportInfo.cs
// Class that implements interface Report, retrieves
// and returns data on weather
using System;
using System.Collections;
using System.IO;         
using System.Net;        
using Weather;           

public class ReportInfo : MarshalByRefObject, Report
{
   private ArrayList cityList; // cities, temperatures, descriptions

   public ReportInfo()
   {
      cityList = new ArrayList();

      // create WebClient to get access to Web page
      WebClient myClient = new WebClient();        

      // get StreamReader for response so we can read page     
      StreamReader input = new StreamReader( myClient.OpenRead(
         "http://iwin.nws.noaa.gov/iwin/us/traveler.html" ) ); 

      string separator1 = "TAV12"; // indicates first batch of cities 
      string separator2 = "TAV13"; // indicates second batch of cities

      // locate separator1 in Web page                                   
      while ( !input.ReadLine().StartsWith( separator1 ) ); // do nothing
      ReadCities( input ); // read the first batch of cities             

      // locate separator2 in Web page                                   
      while ( !input.ReadLine().StartsWith( separator2 ) ); // do nothing
      ReadCities( input ); // read the second batch of cities            

      cityList.Sort(); // sort list of cities by alphabetical order                         
      input.Close(); // close StreamReader to NWS server

      // display the data on the server side
      Console.WriteLine( "Data from NWS Web site:" );

      foreach ( CityWeather city in cityList )
      {
         Console.WriteLine( city );
      } // end foreach
   } // end constructor

   // utility method that reads a batch of cities
   private void ReadCities( StreamReader input )
   {
      // day format and night format
      string dayFormat =                                 
         "CITY            WEA     HI/LO   WEA     HI/LO";
      string nightFormat =                               
         "CITY            WEA     LO/HI   WEA     LO/HI";
      string inputLine = "";

      // locate header that begins weather information
      do                                              
      {                                               
         inputLine = input.ReadLine();                
      } while ( !inputLine.Equals( dayFormat ) &&      
         !inputLine.Equals( nightFormat ) );           

      inputLine = input.ReadLine(); // get first city's data

      // while there are more cities to read
      while ( inputLine.Length > 28 )
      {
         // create CityWeather object for city 
         CityWeather weather = new CityWeather(
            inputLine.Substring( 0, 16 ),      
            inputLine.Substring( 16, 7 ),      
            inputLine.Substring( 23, 7 ) );    

         cityList.Add( weather ); // add to ArrayList         
         inputLine = input.ReadLine(); // get next city's data
      } // end while
   } // end method ReadCities

   // property for getting the cities weather reports
   public ArrayList Reports
   {
      get
      {
         return cityList;
      } // end get
   } // end property Reports
} // end class ReportInfo

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/