// Fig. 24.23: BinaryTreeLibrary2.cs
// Declaration of class TreeNode and class Tree for IComparable
// objects.
using System;

namespace BinaryTreeLibrary2
{
   // class TreeNode declaration
   class TreeNode
   {
      private TreeNode leftNode; // link to left child
      private IComparable data; // data stored in node
      private TreeNode rightNode; // link to right subtree

      // initialize data and make this a leaf node
      public TreeNode( IComparable nodeData )
      {
         data = nodeData;
         leftNode = rightNode = null;  // node has no children
      } // end constructor

      // LeftNode property
      public TreeNode LeftNode
      {
         get
         {
            return leftNode;
         } // end get
         set
         {
            leftNode = value;
         } // end set
      } // end property LeftNode

      // Data property
      public IComparable Data
      {
         get
         {
            return data;
         } // end get
         set
         {
            data = value;
         } // end set
      } // end property Data

      // RightNode property
      public TreeNode RightNode
      {
         get
         {
            return rightNode;
         } // end get
         set
         {
            rightNode = value;
         } // end set
      } // end property Right Node

      // insert TreeNode into Tree that contains nodes;
      // ignore duplicate values
      public void Insert( IComparable insertValue )
      {
         if ( insertValue.CompareTo( data ) < 0 )
         {
            // insert in left subtree
            if ( leftNode == null )
               leftNode = new TreeNode( insertValue );
            else // continue traversing left subtree
               leftNode.Insert( insertValue );
         } // end if
         else if ( insertValue.CompareTo( data ) > 0 )
         {
            // insert in right subtree
            if ( rightNode == null )
               rightNode = new TreeNode( insertValue );
            else // continue traversing right subtree
               rightNode.Insert( insertValue );
         } // end else if
      } // end method Insert
   } // end class TreeNode

   // class Tree declaration
   public class Tree
   {
      private TreeNode root;

      // construct an empty Tree of integers
      public Tree()
      {
         root = null;
      } // end constructor

      // Insert a new node in the binary search tree.
      // If the root node is null, create the root node here.
      // Otherwise, call the insert method of class TreeNode.
      public void InsertNode( IComparable insertValue )
      {
         if ( root == null )
            root = new TreeNode( insertValue );
         else
            root.Insert( insertValue );
      } // end method InsertNode

      // begin preorder traversal
      public void PreorderTraversal()
      {
         PreorderHelper( root );
      } // end method PreorderTraversal

      // recursive method to perform preorder traversal
      private void PreorderHelper( TreeNode node )
      {
         if ( node == null )
            return;

         // output node data
         Console.Write( node.Data + " " );

         // traverse left subtree
         PreorderHelper( node.LeftNode );

         // traverse right subtree
         PreorderHelper( node.RightNode );
      } // end method PreorderHelper

      // begin inorder traversal
      public void InorderTraversal()
      {
         InorderHelper( root );
      } // end method InorderTraversal

      // recursive method to perform inorder traversal
      private void InorderHelper( TreeNode node )
      {
         if ( node == null )
            return;

         // traverse left subtree
         InorderHelper( node.LeftNode );

         // output node data
         Console.Write( node.Data + " " );

         // traverse right subtree
         InorderHelper( node.RightNode );
      } // end method InorderHelper

      // begin postorder traversal
      public void PostorderTraversal()
      {
         PostorderHelper( root );
      } // end method PostorderTraversal

      // recursive method to perform postorder traversal
      private void PostorderHelper( TreeNode node )
      {
         if ( node == null )
            return;

         // traverse left subtree
         PostorderHelper( node.LeftNode );

         // traverse right subtree
         PostorderHelper( node.RightNode );

         // output node data
         Console.Write( node.Data + " " );
      } // end method PostorderHelper

   }  // end class Tree
} // end namespace BinaryTreeLibrary2

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/